import pytest

from h5preserve import H5PreserveFile, new_registry_list

class TestFile(object):
    @pytest.mark.xfail
    def test_context_manager(self, tmpdir, registry_container):
        assert 0
    @pytest.mark.xfail
    def test_create(self, tmpdir, registry_container):
        assert 0
    @pytest.mark.xfail
    def test_close(self, tmpdir, registry_container):
        assert 0

    def test_roundtrip(self, h5py_file):
        assert h5py_file == H5PreserveFile(
            h5py_file, new_registry_list()
        ).h5py_file




import pytest

import numpy as np
from h5py import Group, Dataset, SoftLink, ExternalLink

from h5preserve import RegistryContainer, HardLink, new_registry_list

def is_matching_hdf5_object(new, old):
    """
    Check if two objects have the same hdf5 representation
    """
    if type(new) != type(old):
        return False
    elif isinstance(new, Dataset):
        if new[:] != old[:]:
            return False
        elif new.attrs != old.attrs:
            return False
        return True
    elif isinstance(new, Group):
        if new.keys() != old.keys():
            return False
        elif new.attrs != old.attrs:
            return False
        for key in new.keys():
            if not is_matching_hdf5_object(new[key], old[key]):
                return False
        return True
    return new == old

class TestDump(object):
    def test_hardlink(self, empty_registry):
        registries = RegistryContainer(empty_registry)
        hard_link = HardLink("/example")
        assert hard_link == registries.dump(hard_link)

    def test_h5py_group(self, empty_registry, h5py_file_with_group):
        registries = RegistryContainer(empty_registry)
        group = h5py_file_with_group["example"]
        assert group == registries.dump(group)

    def test_h5py_dataset(self, empty_registry, h5py_file_with_dataset):
        registries = RegistryContainer(empty_registry)
        dataset = h5py_file_with_dataset["example"]
        assert dataset == registries.dump(dataset)

    def test_h5py_soft_link(self, empty_registry):
        registries = RegistryContainer(empty_registry)
        soft_link = SoftLink("/example")
        assert soft_link == registries.dump(soft_link)

    def test_h5py_external_link(self, empty_registry):
        registries = RegistryContainer(empty_registry)
        external_link = ExternalLink("example.hdf5", "/example")
        assert external_link == registries.dump(external_link)

    def test_ndarray(self, empty_registry):
        registries = RegistryContainer(empty_registry)
        ndarray = np.random.rand(1000)
        assert all(ndarray == registries.dump(ndarray))

    @pytest.mark.xfail
    def test_recursion(self, tmpdir, registry_container):
        assert 0

class TestObjToH5preserve(object):
    def test_not_dumpable(self, empty_registry, experiment_data):
        registries = RegistryContainer(empty_registry)
        with pytest.raises(TypeError) as excinfo:
            registries._obj_to_h5preserve(experiment_data)
        assert (
            "<class 'conftest.Experiment'> is not something that can be dumped." == str(excinfo.value)
        )

    def test_no_version(self, experiment_registry, experiment_data):
        registries = RegistryContainer(experiment_registry)
        registries.lock_version(type(experiment_data), 10)
        with pytest.raises(RuntimeError) as excinfo:
            registries._obj_to_h5preserve(experiment_data)
        assert (
            "<class 'conftest.Experiment'> does not have version 10." == str(excinfo.value)
        )

    def test_version(self, experiment_registry, experiment_data):
        registries = RegistryContainer(experiment_registry)
        registries.lock_version(type(experiment_data), 1)
        assert registries._obj_to_h5preserve(experiment_data)._version == 1

    def test_none(self, None_version_experiment_registry, experiment_data):
        registries = RegistryContainer(None_version_experiment_registry)
        assert registries._obj_to_h5preserve(experiment_data)._version == None

    @pytest.mark.xfail
    def test_container_base(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_mapping(self, tmpdir, registry_container):
        assert 0

    def test_invalid_dumper(
        self, invalid_dumper_experiment_registry, experiment_data
    ):
        registries = RegistryContainer(invalid_dumper_experiment_registry)
        with pytest.raises(TypeError) as excinfo:
            registries._obj_to_h5preserve(experiment_data)
        assert (
            "Dumper for Experiment with version 1 returned incorrect type." == str(excinfo.value)
        )

class TestToFile(object):
    @pytest.mark.xfail
    def test_hardlink(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_group(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_dataset(self, tmpdir, registry_container):
        assert 0

    def test_h5py_soft_link(self, h5py_file_with_group):
        soft_link = SoftLink("example")
        registries = new_registry_list()
        registries.to_file(h5py_file_with_group, "alias", soft_link)
        assert h5py_file_with_group.get("alias", getlink=True).path == "example"

    def test_h5py_external_link(self, h5py_file_with_group):
        external = ExternalLink("external.hdf5", "example")
        registries = new_registry_list()
        registries.to_file(h5py_file_with_group, "alias", external)
        assert h5py_file_with_group.get("alias", getlink=True).path == "example"
        assert h5py_file_with_group.get("alias", getlink=True).filename == "external.hdf5"

    @pytest.mark.xfail
    def test_unknown_type(self):
        with pytest.raises(TypeError) as excinfo:
            registries.to_file("fail")
        assert (
            "Unknown h5preserve type str." == str(excinfo.value)
        )



# coding: utf-8
import pytest

from collections import namedtuple

import six

import numpy as np
import h5py
from h5py import File

from h5preserve import (
    Registry, RegistryContainer, new_registry_list, GroupContainer,
    DatasetContainer
)
from h5preserve.additional_registries import (
    none_python_registry, builtin_numbers_registry,
    builtin_text_registry
)

### Classes for testng ###
class Experiment(object):
    def __init__(self, data, time_started):
        self.data = data
        self.time_started = time_started

    def __eq__(self, other):
        return (
            all(self.data == other.data) and
            self.time_started == other.time_started
        )

Experiments = namedtuple("Experiments", ["runs", "name"])

def _better_eq(self, other):
    if not isinstance(other, self.__class__):
        return False
    elif len(self) != len(other):
        return False
    for i in range(len(self)):
        result = self[i] == other[i]
        try:
            bool(result)
        except ValueError:
            if not result.all():
                return False
        else:
            if not result:
                return False
    return True


InternalData = namedtuple("InternalData", [
    "derivs", "params", "angles", "v_r_normal", "v_phi_normal", "rho_normal",
    "v_r_taylor", "v_phi_taylor", "rho_taylor",
])
InternalData.__eq__ = _better_eq

InitialConditions = namedtuple("InitialConditions", [
    "norm_kepler_sq", "c_s", "eta_O", "eta_A", "eta_H", "beta", "init_con",
    "angles",
])
InitialConditions.__eq__ = _better_eq

Solution = namedtuple("Solution", [
    "angles", "solution", "flag", "coordinate_system",
    "internal_data", "initial_conditions", "t_roots", "y_roots",
])
Solution.__eq__ = _better_eq
###


@pytest.fixture
def empty_registry():
    return Registry("empty registry")

@pytest.fixture
def frozen_empty_registry():
    registry = empty_registry()
    registry.freeze()
    return registry

@pytest.fixture
def experiment_registry():
    registry = Registry("experiment")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return DatasetContainer(
            data = experiment.data,
            attrs = {
                "time started": experiment.time_started
            }
        )

    @registry.loader("Experiment", version=1)
    def _exp_load(dataset):
        return Experiment(
            data=dataset["data"],
            time_started=dataset["attrs"]["time started"]
        )

    return registry

@pytest.fixture
def experiment_registry_as_group():
    registry = Registry("experiment")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return GroupContainer(
            dataset=DatasetContainer(data = experiment.data),
            attrs = {
                "time started": experiment.time_started
            }
        )

    @registry.loader("Experiment", version=1)
    def _exp_load(dataset):
        return Experiment(
            data=dataset["dataset"]["data"],
            time_started=dataset.attrs["time started"]
        )

    return registry

@pytest.fixture
def experiment_experiments_registry():
    registry = Registry("experiments")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return GroupContainer(
            dataset=DatasetContainer(data = experiment.data),
            attrs = {
                "time started": experiment.time_started
            }
        )

    @registry.loader("Experiment", version=1)
    def _exp_load(dataset):
        return Experiment(
            data=dataset["dataset"]["data"],
            time_started=dataset.attrs["time started"]
        )

    @registry.dumper(Experiments, "Experiments", version=1)
    def _exp_dump(experiments):
        return GroupContainer(
            runs=GroupContainer(**experiments.runs),
            attrs = {
                "name": experiments.name
            }
        )

    @registry.loader("Experiments", version=1)
    def _exp_load(group):
        return Experiments(
            runs={run: exp for run, exp in group["runs"].items()},
            name=group.attrs["name"]
        )

    return registry

@pytest.fixture
def invalid_dumper_experiment_registry():
    registry = Registry("incorrect dumper experiment")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return (
            experiment.data, {
                "time started": experiment.time_started
            }
        )

    return registry

@pytest.fixture
def invalid_loader_experiment_registry():
    registry = Registry("incorrect loader experiment")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return DatasetContainer(
            data = experiment.data,
            attrs = {
                "time started": experiment.time_started
            }
        )

    @registry.loader("Experiment", version=1)
    def _exp_load(dataset):
        return Experiment(
            data=dataset["data"],
        )

    return registry

@pytest.fixture
def None_version_experiment_registry():
    registry = Registry("None experiment")

    @registry.dumper(Experiment, "Experiment", version=None)
    def _exp_dump(experiment):
        return DatasetContainer(
            data = experiment.data,
            attrs = {
                "time started": experiment.time_started
            }
        )

    @registry.loader("Experiment", version=None)
    def _exp_load(dataset):
        return Experiment(
            data=dataset["data"],
            time_started=dataset["attrs"]["time started"]
        )

    return registry

@pytest.fixture
def no_loader_experiment_registry():
    registry = Registry("experiment")

    @registry.dumper(Experiment, "Experiment", version=1)
    def _exp_dump(experiment):
        return DatasetContainer(
            data = experiment.data,
            attrs = {
                "time started": experiment.time_started
            }
        )

    return registry

@pytest.fixture
def frozen_experiment_registry():
    registry = experiment_registry()
    registry.freeze()
    return registry

@pytest.fixture
def experiment_data():
    return Experiment(
        data=np.random.rand(100),
        time_started="1970-01-01 00:00:00"
    )

@pytest.fixture
def multi_experiment_data():
    return Experiments(runs={
        "1": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-01 00:00:00"
        ),
        "2": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-02 00:00:00"
        ),
        "3": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-03 00:00:00"
        ),
        "4": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-04 00:00:00"
        ),
        "5": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-05 00:00:00"
        ),
        "6": Experiment(
            data=np.random.rand(100),
            time_started="1970-01-06 00:00:00"
        ),
    }, name="test"
    )

@pytest.fixture
def solution_registry():
    registry = Registry("solution")

    @registry.dumper(InternalData, "InternalData", version=1)
    def _internal_dump(internal_data):
        return GroupContainer(
            derivs = internal_data.derivs,
            params = internal_data.params,
            angles = internal_data.angles,
            v_r_normal = internal_data.v_r_normal,
            v_phi_normal = internal_data.v_phi_normal,
            rho_normal = internal_data.rho_normal,
            v_r_taylor = internal_data.v_r_taylor,
            v_phi_taylor = internal_data.v_phi_taylor,
            rho_taylor = internal_data.rho_taylor,
        )

    @registry.loader("InternalData", version=1)
    def _internal_load(group):
        return InternalData(
            derivs = group["derivs"]["data"],
            params = group["params"]["data"],
            angles = group["angles"]["data"],
            v_r_normal = group["v_r_normal"]["data"],
            v_phi_normal = group["v_phi_normal"]["data"],
            rho_normal = group["rho_normal"]["data"],
            v_r_taylor = group["v_r_taylor"]["data"],
            v_phi_taylor = group["v_phi_taylor"]["data"],
            rho_taylor = group["rho_taylor"]["data"],
        )

    @registry.dumper(InitialConditions, "InitialConditions", version=1)
    def _initial_dump(initial_conditions):
        return GroupContainer(
            attrs={
                "norm_kepler_sq": initial_conditions.norm_kepler_sq,
                "c_s": initial_conditions.c_s,
                "eta_O": initial_conditions.eta_O,
                "eta_A": initial_conditions.eta_A,
                "eta_H": initial_conditions.eta_H,
                "beta": initial_conditions.beta,
                "init_con": initial_conditions.init_con,
            }, angles = initial_conditions.angles,
        )

    @registry.loader("InitialConditions", version=1)
    def _initial_load(group):
        return InitialConditions(
            norm_kepler_sq = group.attrs["norm_kepler_sq"],
            c_s = group.attrs["c_s"],
            eta_O = group.attrs["eta_O"],
            eta_A = group.attrs["eta_A"],
            eta_H = group.attrs["eta_H"],
            beta = group.attrs["beta"],
            init_con = group.attrs["init_con"],
            angles = group["angles"]["data"],
        )

    @registry.dumper(Solution, "Solution", version=1)
    def _solution_dumper(solution):
        return GroupContainer(
            attrs={
                "flag": solution.flag,
                "coordinate_system": solution.coordinate_system,
            },
            angles = solution.angles,
            solution = solution.solution,
            internal_data = solution.internal_data,
            initial_conditions = solution.initial_conditions,
            t_roots = solution.t_roots,
            y_roots = solution.y_roots,
        )

    @registry.loader("Solution", version=1)
    def _solution_loader(group):
        return Solution(
            flag = group.attrs["flag"],
            coordinate_system = group.attrs["coordinate_system"],
            angles = group["angles"]["data"],
            solution = group["solution"]["data"],
            t_roots = group["t_roots"]["data"],
            y_roots = group["y_roots"]["data"],
            internal_data = group["internal_data"],
            initial_conditions = group["initial_conditions"],
        )

    return registry

@pytest.fixture
def internal_data_data():
    return InternalData(
        derivs = np.random.rand(1000, 8),
        params = np.random.rand(1000, 8),
        angles = np.random.rand(1000),
        v_r_normal = np.random.rand(1000),
        v_phi_normal = np.random.rand(1000),
        rho_normal = np.random.rand(1000),
        v_r_taylor = np.random.rand(1000),
        v_phi_taylor = np.random.rand(1000),
        rho_taylor = np.random.rand(1000),
    )

@pytest.fixture
def initial_conditions_data():
    return InitialConditions(
        norm_kepler_sq = 10,
        c_s = 1.0,
        eta_O = 0.3,
        eta_A = 0.001,
        eta_H = 5e-15,
        beta = 5/3,
        init_con = np.random.rand(8),
        angles = np.random.rand(1000),
    )

@pytest.fixture
def solution_data():
    return Solution(
        flag = 0,
        coordinate_system = "some thing",
        angles = np.random.rand(1000),
        solution = np.random.rand(1000, 8),
        t_roots = np.random.rand(15),
        y_roots = np.random.rand(15,8),
        internal_data = internal_data_data(),
        initial_conditions = initial_conditions_data(),
    )

@pytest.fixture(params=[
    (experiment_registry(), experiment_data()),
    (experiment_registry_as_group(), experiment_data()),
    (frozen_experiment_registry(), experiment_data()),
    (None_version_experiment_registry(), experiment_data()),
    (solution_registry(), internal_data_data()),
    (solution_registry(), initial_conditions_data()),
    (solution_registry(), solution_data()),
    (builtin_numbers_registry, 1),
    (builtin_numbers_registry, 1.0),
    (builtin_text_registry, b"abcd"),
    (builtin_text_registry, u"abcd"),
    (experiment_experiments_registry(), multi_experiment_data()),
])
def obj_registry(request):
    return {
        "registries": RegistryContainer(request.param[0]),
        "dumpable_object": request.param[1]
    }

@pytest.fixture(params=[
    (experiment_registry(), experiment_data()),
    (frozen_experiment_registry(), experiment_data()),
    (None_version_experiment_registry(), experiment_data()),
    (solution_registry(), internal_data_data()),
    (solution_registry(), initial_conditions_data()),
    (solution_registry(), solution_data()),
    (builtin_numbers_registry, 1),
    (builtin_numbers_registry, 1.0),
    (frozen_empty_registry(), 1),
    (frozen_empty_registry(), 1.0),
    (builtin_text_registry, b"abcd"),
    (builtin_text_registry, u"abcd"),
    (frozen_empty_registry(), b"abcd"),
    (frozen_empty_registry(), u"abcd"),
])
def obj_registry_with_defaults(request):
    return {
        "registries": new_registry_list(request.param[0]),
        "dumpable_object": request.param[1]
    }

if hasattr(h5py, "Empty"):

    @pytest.fixture(params=[
        (none_python_registry, None),
    ])
    def obj_registry_with_none(request):
        return {
            "registries": RegistryContainer(request.param[0]),
            "dumpable_object": request.param[1]
        }

    @pytest.fixture(params=[
        (none_python_registry, None),
        (frozen_empty_registry(), None),
    ])
    def obj_registry_with_none_with_defaults(request):
        return {
            "registries": new_registry_list(request.param[0]),
            "dumpable_object": request.param[1]
        }

@pytest.fixture
def h5py_file(tmpdir):
    file = File(str(tmpdir.join("test.hdf5")))
    def fin():
        file.close()
    return file

@pytest.fixture
def h5py_file_with_group(h5py_file):
    h5py_file.create_group("example")
    return h5py_file

@pytest.fixture
def h5py_file_with_dataset(h5py_file):
    h5py_file.create_dataset("example", data=np.random.rand(1000))
    return h5py_file

@pytest.fixture
def h5py_file_with_experiment(h5py_file):
    h5py_file.create_dataset("example", data=np.random.rand(1000))
    h5py_file["example"].attrs.update()
    return h5py_file

@pytest.fixture
def h5preserve_experiment_repr():
    return DatasetContainer(
        attrs={
            "time started": "2000-01-01",
            "_h5preserve_namespace": "experiment",
            "_h5preserve_label": "Experiment",
            "_h5preserve_version": 1,
        },
        data=np.random.rand(1000)
    )



# coding: utf-8
"""
Additional registries for h5preserve
"""
import six

from numpy import asarray
import h5py

from . import Registry, DatasetContainer


def as_dataset_dumper(obj):
    # pylint: disable=missing-docstring
    return DatasetContainer(data=obj)


def as_dataset_loader(dataset):
    # pylint: disable=missing-docstring
    return dataset["data"]


none_python_registry = Registry("Python: None")
builtin_numbers_registry = Registry("Python: builtin numbers")
builtin_text_registry = Registry("Python: builtin text")
sequence_as_dataset_registry = Registry("Python: sequence as dataset")

if hasattr(h5py, "Empty"):
    # pylint: disable=missing-docstring,unused-argument,no-member
    @none_python_registry.dumper(type(None), "None", version=None)
    def _none_dumper(none):
        return DatasetContainer(dtype=float)

    @none_python_registry.loader("None", version=None)
    def _none_loader(empty):
        return None

builtin_numbers_registry.dumper(int, "int", version=None)(as_dataset_dumper)
builtin_numbers_registry.loader("int", version=None)(as_dataset_loader)
builtin_numbers_registry.dumper(float, "float", version=None)(
    as_dataset_dumper
)
builtin_numbers_registry.loader("float", version=None)(as_dataset_loader)

if six.PY2:
    builtin_text_registry.dumper(str, "ascii", version=None)(
        as_dataset_dumper
    )
    builtin_text_registry.dumper(unicode, "text", version=None)(
        as_dataset_dumper
    )
else:
    builtin_text_registry.dumper(bytes, "ascii", version=None)(
        as_dataset_dumper
    )
    builtin_text_registry.dumper(str, "text", version=None)(
        as_dataset_dumper
    )
builtin_text_registry.loader("ascii", version=None)(as_dataset_loader)
builtin_text_registry.loader("text", version=None)(as_dataset_loader)


@sequence_as_dataset_registry.dumper(list, "list", version=None)
def _list_dumper(l):
    # pylint: disable=missing-docstring
    return DatasetContainer(data=asarray(l))


@sequence_as_dataset_registry.loader("list", version=None)
def _list_loader(dataset):
    # pylint: disable=missing-docstring
    return list(dataset)


@sequence_as_dataset_registry.dumper(tuple, "tuple", version=None)
def _tuple_dumper(t):
    # pylint: disable=missing-docstring
    return DatasetContainer(data=asarray(t))


@sequence_as_dataset_registry.loader("tuple", version=None)
def _tuple_loader(dataset):
    # pylint: disable=missing-docstring
    return tuple(dataset)


none_python_registry.freeze()
builtin_numbers_registry.freeze()
builtin_text_registry.freeze()
sequence_as_dataset_registry.freeze()



import pytest

import h5py
from h5preserve import open as hp_open, H5PreserveFile

@pytest.mark.roundtrip
def test_roundtrip(tmpdir, obj_registry):
    tmpfile = str(tmpdir.join("test_roundtrip.h5"))
    with hp_open(tmpfile, registries=obj_registry["registries"]) as f:
        f["first"] = obj_registry["dumpable_object"]

    with hp_open(tmpfile, registries=obj_registry["registries"]) as f:
        assert f["first"] == obj_registry["dumpable_object"]

@pytest.mark.roundtrip
def test_roundtrip_without_open(tmpdir, obj_registry):
    tmpfile = str(tmpdir.join("test_roundtrip.h5"))
    with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry["registries"]) as f:
        f["first"] = obj_registry["dumpable_object"]

    with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry["registries"]) as f:
        assert f["first"] == obj_registry["dumpable_object"]

@pytest.mark.roundtrip
def test_roundtrip_with_defaults(tmpdir, obj_registry_with_defaults):
    obj_registry = obj_registry_with_defaults
    tmpfile = str(tmpdir.join("test_roundtrip.h5"))
    with hp_open(tmpfile, registries=obj_registry["registries"]) as f:
        f["first"] = obj_registry["dumpable_object"]

    with hp_open(tmpfile, registries=obj_registry["registries"]) as f:
        assert f["first"] == obj_registry["dumpable_object"]

@pytest.mark.roundtrip
def test_roundtrip_without_open_with_defaults(tmpdir, obj_registry_with_defaults):
    obj_registry = obj_registry_with_defaults
    tmpfile = str(tmpdir.join("test_roundtrip.h5"))
    with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry["registries"]) as f:
        f["first"] = obj_registry["dumpable_object"]

    with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry["registries"]) as f:
        assert f["first"] == obj_registry["dumpable_object"]

if hasattr(h5py, "Empty"):
    @pytest.mark.roundtrip
    def test_roundtrip_with_none(tmpdir, obj_registry_with_none):
        tmpfile = str(tmpdir.join("test_roundtrip.h5"))
        with hp_open(tmpfile, registries=obj_registry_with_none["registries"]) as f:
            f["first"] = obj_registry_with_none["dumpable_object"]

        with hp_open(tmpfile, registries=obj_registry_with_none["registries"]) as f:
            assert f["first"] == obj_registry_with_none["dumpable_object"]

    @pytest.mark.roundtrip
    def test_roundtrip_without_open_with_none(tmpdir, obj_registry_with_none):
        tmpfile = str(tmpdir.join("test_roundtrip.h5"))
        with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry_with_none["registries"]) as f:
            f["first"] = obj_registry_with_none["dumpable_object"]

        with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry_with_none["registries"]) as f:
            assert f["first"] == obj_registry_with_none["dumpable_object"]

    @pytest.mark.roundtrip
    def test_roundtrip_with_defaults_with_none(tmpdir, obj_registry_with_none_with_defaults):
        obj_registry_with_none = obj_registry_with_none_with_defaults
        tmpfile = str(tmpdir.join("test_roundtrip.h5"))
        with hp_open(tmpfile, registries=obj_registry_with_none["registries"]) as f:
            f["first"] = obj_registry_with_none["dumpable_object"]

        with hp_open(tmpfile, registries=obj_registry_with_none["registries"]) as f:
            assert f["first"] == obj_registry_with_none["dumpable_object"]

    @pytest.mark.roundtrip
    def test_roundtrip_without_open_with_defaults_with_none(tmpdir, obj_registry_with_none_with_defaults):
        obj_registry_with_none = obj_registry_with_none_with_defaults
        tmpfile = str(tmpdir.join("test_roundtrip.h5"))
        with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry_with_none["registries"]) as f:
            f["first"] = obj_registry_with_none["dumpable_object"]

        with H5PreserveFile(h5py.File(tmpfile), registries=obj_registry_with_none["registries"]) as f:
            assert f["first"] == obj_registry_with_none["dumpable_object"]



import pytest

class MappingTestingClass(object):
    def test_getitem(self):
        raise NotImplementedError("Missing __getitem__ test")
    def test_iter(self):
        raise NotImplementedError("Missing __iter__ test")
    def test_len(self):
        raise NotImplementedError("Missing __len__ test")

class MutableMappingTestingClass(MappingTestingClass):
    def test_setitem(self):
        raise NotImplementedError("Missing __setitem__ test")
    def test_delitem(self):
        raise NotImplementedError("Missing __delitem__ test")

class SequenceTestingClass(object):
    def test_getitem(self):
        raise NotImplementedError("Missing __getitem__ test")
    def test_len(self):
        raise NotImplementedError("Missing __len__ test")

class MutableSequenceTestingClass(MappingTestingClass):
    def test_setitem(self):
        raise NotImplementedError("Missing __setitem__ test")
    def test_delitem(self):
        raise NotImplementedError("Missing __delitem__ test")
    def test_insert(self):
        raise NotImplementedError("Missing __insert__ test")

@pytest.mark.xfail
class TestRegistryContainer(MutableSequenceTestingClass):
    pass

@pytest.mark.xfail
class TestGroupContainer(MutableMappingTestingClass):
    pass

@pytest.mark.xfail
class TestDatasetContainer(MutableMappingTestingClass):
    pass

@pytest.mark.xfail
class TestH5PreserveGroup(MutableMappingTestingClass):
    pass



# coding: utf-8
"""
h5preserve is a thin wrapper around h5py, providing easier serialisation of
native python types.

:copyright: (c) 2016 James Tocknell
:license: 3-clause BSD
"""
from collections import (
    MutableMapping, namedtuple, defaultdict, MutableSequence,
)
from warnings import warn

import six

from numpy import ndarray
import h5py

# versioneer stuff
from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

ALLOWED_DATASET_KEYS = {
    "attrs", "shape", "dtype", "data", "chunks", "maxshape", "fillvalue",
    "size", "compression", "compression_opts", "scaleoffset", "shuffle",
    "fletcher32", "fillvalue", "track_times",
}

EXTERNAL_DUMPED_TYPES = {
    h5py.Group,
    h5py.Dataset,
    h5py.SoftLink,
    h5py.ExternalLink,
    ndarray,
}

H5PY_ATTR_WRITABLE_TYPES = {
    int,
    float,
    list,
    tuple,
    ndarray,
}
H5PY_ATTR_WRITABLE_TYPES.add(six.text_type)
H5PY_ATTR_WRITABLE_TYPES.add(six.binary_type)

H5PRESERVE_ATTR_NAMESPACE = "_h5preserve_namespace"
H5PRESERVE_ATTR_LABEL = "_h5preserve_label"
H5PRESERVE_ATTR_VERSION = "_h5preserve_version"

UNKNOWN_NAMESPACE = "Unknown namespace {}."
UNSUPPORTED_H5PY_TYPE = "Unsupported h5py type {}."
UNKNOWN_H5PRESERVE_TYPE = "Unknown h5preserve type {}."
NOT_DUMPABLE = "{} is not something that can be dumped."
NO_VERSION = "{} does not have version {}."
INVALID_DUMPER = "Dumper for {} with version {} returned incorrect type."
NOT_LOADABLE = "{} is not something that can be loaded."
LABEL_NOT_IN_NAMESPACE = "Label {} not in namespace {}."
NO_SUITABLE_LOADER = "Cannot find suitable loader for label {} with version {}"
INVALID_DATASET_OPTION = "{} is not a valid dataset option."
NO_PATH = "No path defined for hard link."
ATTR_NOT_DUMPED = "Attribute {}={} has not been dumped."

_DumperMap = namedtuple("_DumperMap", "label func")


class RegistryContainer(MutableSequence):
    """
    Ordered container of registries which manages interaction with the hdf5
    file.

    Parameters
    ----------
    *registries : list of Registry
        the list of registries to be associated with this container
    """
    def __init__(self, *registries):
        self._version_lock = {}
        self._registries = {}
        self._indexed_registries = []
        if registries:
            self.extend(registries)

    def __getitem__(self, index):
        return self._indexed_registries[index]

    def __setitem__(self, index, item):
        self._indexed_registries[index] = item.name
        self._registries[item.name] = item

    def __delitem__(self, index):
        name = self._indexed_registries[index]
        del self._registries[name]
        del self._indexed_registries[index]

    def __len__(self):
        return len(self._registries)

    def insert(self, index, item):
        self._registries[item.name] = item
        self._indexed_registries.insert(index, item.name)

    def from_file(self, h5py_obj):
        """
        Return an representation of a hdf5 object from a hdf5 file

        Parameters
        ----------
        h5py_obj : a ``h5py`` object, e.g. group, dataset
        """
        namespace = h5py_obj.attrs.get(H5PRESERVE_ATTR_NAMESPACE)
        if namespace is None:
            if isinstance(h5py_obj, h5py.Group):
                return H5PreserveGroup(h5py_group=h5py_obj, registries=self)
            warn(
                "No type information about object, returning native h5py"
                "object"
            )
            return h5py_obj
        elif namespace in self._registries:
            return self._h5py_to_h5preserve(h5py_obj)
        raise RuntimeError(UNKNOWN_NAMESPACE.format(namespace))

    def _h5py_to_h5preserve(self, h5py_obj):
        """
        convert h5py object to h5preserve representation
        """
        attrs = dict(h5py_obj.attrs)
        if isinstance(h5py_obj, h5py.Group):
            return GroupContainer(
                attrs,
                **{
                    name: self._h5py_to_h5preserve(item)
                    for name, item in h5py_obj.items()
                }
            )
        elif isinstance(h5py_obj, h5py.Dataset):
            return DatasetContainer(
                attrs,
                data=h5py_obj[()],
                shape=h5py_obj.shape,
                fillvalue=h5py_obj.fillvalue,
                dtype=h5py_obj.dtype,
                size=h5py_obj.size,
            )
        raise TypeError(UNSUPPORTED_H5PY_TYPE.format(type(h5py_obj)))

    def to_file(self, h5py_group, key, val):
        """
        Dump h5preserve object to hdf5 file

        Parameters
        ----------
        h5py_group : ``h5py.Group``
            the group to add the object to
        key : string
            the name for the object
        val
            the object to add
        """
        if isinstance(val, ContainerBase):
            self._write_containers_to_file(h5py_group, key, val)
        elif isinstance(val, HardLink):
            if val.h5py_obj is None:
                # pylint: disable=protected-access
                val._set_file(h5py_group.file)
            h5py_group[key] = val.h5py_obj
        elif isinstance(val, h5py.SoftLink):
            h5py_group[key] = val
        elif isinstance(val, h5py.ExternalLink):
            h5py_group[key] = val
        elif isinstance(val, ndarray):
            h5py_group[key] = val
        else:
            raise TypeError(
                UNKNOWN_H5PRESERVE_TYPE.format(type(val))
            )

    def _write_containers_to_file(self, h5py_group, key, val):
        """
        Write instances of ContainerBase to an hdf5 file
        """
        for item_name, item in val.attrs.items():
            if not isinstance(item, tuple(H5PY_ATTR_WRITABLE_TYPES)):
                raise TypeError(ATTR_NOT_DUMPED.format(item_name, item))
        if isinstance(val, GroupContainer):
            new_obj = h5py_group.create_group(key)
            new_obj.attrs.update(val.attrs)
            for obj_name, obj_val in val.items():
                self.to_file(new_obj, obj_name, obj_val)
        elif isinstance(val, DatasetContainer):
            new_obj = h5py_group.create_dataset(
                key, **val
            )
            new_obj.attrs.update(val.attrs)
        # pylint: disable=protected-access
        if val._label is not None:
            new_obj.attrs[H5PRESERVE_ATTR_LABEL] = val._label
        if val._namespace is not None:
            new_obj.attrs[H5PRESERVE_ATTR_NAMESPACE] = val._namespace
        if val._version is not None:
            new_obj.attrs[H5PRESERVE_ATTR_VERSION] = val._version
        # pylint: enable=protected-access

    def dump(self, obj):
        """
        Dump native python object to h5preserve representation

        Parameters
        ----------
        obj
            the object to dump
        """
        # pylint: disable=unidiomatic-typecheck
        if isinstance(obj, HardLink):
            return obj
        elif type(obj) in EXTERNAL_DUMPED_TYPES:
            return obj
        # pylint: enable=unidiomatic-typecheck
        converted_obj = self._obj_to_h5preserve(obj)
        if isinstance(converted_obj, tuple(RECURSIVE_DUMPING_TYPES)):
            for key, val in converted_obj.items():
                converted_obj[key] = self.dump(val)
        return converted_obj

    def _obj_to_h5preserve(self, obj):
        """convert python object to h5preserve representation"""
        if isinstance(obj, ContainerBase):
            return obj
        val_type = type(obj)
        for registry in self:
            # pylint: disable=protected-access
            if val_type in self._registries[registry].dumpers:
                namespace = registry
                dumpers = self._registries[registry].dumpers[val_type]
                break
        else:
            raise TypeError(NOT_DUMPABLE.format(val_type))

        # pylint: disable=protected-access
        if val_type in self._version_lock:
            version = self._version_lock[val_type]
        # pylint: enable=protected-access
            try:
                label, dumper = dumpers[version]
            except KeyError:
                raise RuntimeError(NO_VERSION.format(val_type, version))
        else:
            if None in dumpers:
                version = None
            else:
                version = sorted(dumpers, reverse=True)[0]
            label, dumper = dumpers[version]
        obj = dumper(obj)
        if isinstance(obj, ContainerBase):
            # pylint: disable=protected-access
            obj._namespace = namespace
            obj._label = label
            obj._version = version
            # pylint: enable=protected-access
            return obj
        raise TypeError(INVALID_DUMPER.format(label, version))

    def load(self, obj):
        """
        Load native python object from h5preserve representation

        Parameters
        ----------
        obj
            the object to dump
        """
        if not isinstance(obj, ContainerBase):
            raise TypeError(NOT_LOADABLE.format(type(obj)))

        if isinstance(obj, GroupContainer):
            new_obj = GroupContainer(
                attrs=obj.attrs,
                **{key: self.load(item) for key, item in obj.items()}
            )
            # pylint: disable=protected-access
            new_obj._namespace = obj._namespace
            new_obj._label = obj._label
            new_obj._version = obj._version
            # pylint: enable=protected-access
            obj = new_obj

        # pylint: disable=protected-access
        if obj._namespace is None:
            return obj
        elif obj._namespace not in self:
            raise RuntimeError(UNKNOWN_NAMESPACE.format(obj._namespace))
        # pylint: enable=protected-access

        return self._get_loader(obj)(obj)

    def _get_loader(self, obj):
        """
        get the loader for obj
        """
        # pylint: disable=protected-access
        loaders = self._registries[obj._namespace].loaders
        if obj._label not in loaders:
            raise RuntimeError(
                LABEL_NOT_IN_NAMESPACE.format(obj._label, obj._namespace)
            )
        loaders = loaders[obj._label]
        if obj._version is None and None in loaders:
            return loaders[None]
        elif all in loaders:
            return loaders[all]
        elif obj._version in loaders:
            return loaders[obj._version]
        try:
            return loaders[any]
        except KeyError:
            raise RuntimeError(
                NO_SUITABLE_LOADER.format(obj._label, obj._version)
            )

    def lock_version(self, cls, version):
        """
        Lock output version for a specific class

        Parameters
        ----------
        cls : any class
            the class to lock the version of
        version : integer, any, all, None
            the version which will always be used
        """
        self._version_lock[cls] = version


class ContainerBase(MutableMapping):
    # pylint: disable=abstract-method,missing-docstring
    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}
        self._namespace = attrs.pop(H5PRESERVE_ATTR_NAMESPACE, None)
        self._label = attrs.pop(H5PRESERVE_ATTR_LABEL, None)
        self._version = attrs.pop(H5PRESERVE_ATTR_VERSION, None)
        self.attrs = attrs


class GroupContainer(ContainerBase):
    """
    Representation of an hdf5 group for use in h5preserve.

    Parameters
    ----------
    attrs : Mapping
        mapping containing the attributes of the group
    **kwargs
        datasets or subgroups to add to the group
    """
    def __init__(self, attrs=None, **kwargs):
        super(GroupContainer, self).__init__(attrs)
        self._group_members = {}
        self.update(kwargs)

    def __getitem__(self, key):
        return self._group_members[key]

    def __setitem__(self, key, val):
        self._group_members[key] = val

    def __delitem__(self, key):
        self._group_members.__delitem__(key)

    def __iter__(self):
        return iter(self._group_members)

    def __len__(self):
        return len(self._group_members)

    def __repr__(self):
        return "GroupContainer(attrs={attrs!r}, {group_items})".format(
            attrs=self.attrs,
            group_items=", ".join(
                "{key!r}={val!r}".format(key=key, val=val)
                for key, val in self._group_members.items()
                if val is not None
            )
        )


class DatasetContainer(ContainerBase):
    """
    Representation of an hdf5 dataset for use in h5preserve.

    Parameters
    ----------
    attrs : Mapping
        mapping containing the attributes of the group
    **kwargs
        properties of the group, which get passed to create group
    """
    def __init__(self, attrs=None, **kwargs):
        super(DatasetContainer, self).__init__(attrs)
        self._dataset_members = {}
        self.update(kwargs)

    def __getitem__(self, key):
        if key == "attrs":
            return self.attrs
        return self._dataset_members[key]

    def __setitem__(self, key, val):
        if key in ALLOWED_DATASET_KEYS:
            self._dataset_members[key] = val
        else:
            raise TypeError(INVALID_DATASET_OPTION.format(key))

    def __delitem__(self, key):
        self._dataset_members.__delitem__(key)

    def __iter__(self):
        return iter(self._dataset_members)

    def __len__(self):
        return len(self._dataset_members)

    def __repr__(self):
        return "DatasetContainer(attrs={attrs!r}, {dataset})".format(
            attrs=self.attrs,
            dataset=", ".join(
                "{key!r}={val!r}".format(key=key, val=val)
                for key, val in self._dataset_members.items()
                if val is not None
            )
        )


class Registry(object):
    """
    Register of functions for converting between hdf5 and python.

    This is the core of h5preserve, containing the information about how to
    convert to and from hdf5 files, what version to use, and the namespace of
    created data.

    Parameters
    ----------
    name : string
        name of registry for identification purposes
    """
    def __init__(self, name):
        self._name = name
        self._frozen = False
        self.dumpers = defaultdict(dict)
        self.loaders = defaultdict(dict)

    @property
    def name(self):
        """str: name of the registry"""
        return self._name

    def freeze(self):
        """
        Freeze the registry, preventing further changes to the registry.
        """
        self._frozen = True

    def dumper(self, cls, label, version):
        """
        Decorator function to create a dumper function.

        Parameters
        ----------
        cls : any class
            the class which this dumper operates on
        label : string
            the label or tag associated with this class
        version : integer, None
            The version of the output that this function returns.
        """
        def add_dumper(new_dumper):
            # pylint: disable=missing-docstring
            self.dumpers[cls][version] = _DumperMap(
                label=label, func=new_dumper
            )
        return add_dumper

    def loader(self, label, version):
        """
        Decorator function to create a loader function.

        Parameters
        ----------
        label : string
            the label or tag associated with this class
        version : integer, any, all, None
            The version of the output that this function reads.
        """
        def add_loader(new_loader):
            # pylint: disable=missing-docstring
            self.loaders[label][version] = new_loader
        return add_loader


class H5PreserveGroup(MutableMapping):
    """
    Thin wrapper around :class:`h5py.Group` to automatically use h5preserve
    when accessing the group contents.

    Parameters
    ----------
    h5py_group : ``h5py.Group``
    registries : RegistryContainer
        the collection of registries that you want to use to read from the hdf5
        file
    """
    def __init__(self, h5py_group, registries):
        self._h5py_group = h5py_group
        self.registries = registries

    def __getitem__(self, key):
        return self.registries.load(
            self.registries.from_file(self._h5py_group[key])
        )

    def __setitem__(self, key, val):
        self.registries.to_file(
            self._h5py_group,
            key,
            self.registries.dump(val)
        )

    def __delitem__(self, key):
        del self._h5py_group[key]

    def __iter__(self):
        for key in self._h5py_group:
            yield key

    def __len__(self):
        return len(self._h5py_group)

    @property
    def h5py_group(self):
        """
        h5py.Group: the instance of ``h5py.Group`` which ``H5PreserveGroup``
        wraps
        """
        return self._h5py_group

    def create_group(self, name):
        """
        Creates a new group in the associated hdf5 file

        Parameters
        ----------
        name : string, or other identifier accepted by h5py
            name of the new group

        Returns
        -------
        H5PreserveGroup
            The new group wrapped by H5PreserveGroup
        """
        return H5PreserveGroup(
            self._h5py_group.create_group(name), self.registries
        )

    def require_group(self, name):
        """
        Returns the group associated with ``name``, creating it if necessary.

        Parameters
        ----------
        name : string, or other identifier accepted by h5py
            name of the desired group

        Returns
        -------
        H5PreserveGroup
            The group wrapped by H5PreserveGroup
        """
        return H5PreserveGroup(
            self._h5py_group.require_group(name), self.registries
        )


class H5PreserveFile(H5PreserveGroup):
    """
    Thin wrapper around ``h5py.File`` to automatically use h5preserve when
    accessing the file contents.

    Acts like ``h5preserve.H5PreserveGroup``, but allows access to the
    associated ``h5py.File`` instance via ``h5py_file``.

    Parameters
    ----------
    h5py_file : a ``h5py.File``
        the hdf5 file to wrap
    registries : RegistryContainer
        the collection of registries that you want to use to read from the hdf5
        file
    """
    def __init__(self, h5py_file, registries):
        self._h5py_file = h5py_file
        super(H5PreserveFile, self).__init__(
            h5py_group=self._h5py_file["/"],
            registries=registries
        )

    def close(self):
        # pylint: disable=missing-docstring
        self._h5py_file.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()
        return False

    @property
    def h5py_file(self):
        """
        h5py.File: the instance of ``h5py.File`` which ``H5PreserveFile`` wraps
        """
        return self._h5py_file


class HardLink(object):
    # pylint: disable=too-few-public-methods
    """
    Represent a h5py hard link to be created via h5preserve.

    Parameters
    ----------
    obj : string, h5py.Group or h5py.Dataset
        the h5py object that the hard link points to, can either be an h5py
        object, or a string with the absolute path of the object
    """
    def __init__(self, obj):
        if isinstance(obj, str):
            self._path = obj
            self._h5py_obj = None
        else:
            self._h5py_obj = obj
            self._path = None

    @property
    def h5py_obj(self):
        """
        h5py.Group or h5py.Dataset: the object which the hard link will point
        to
        """
        return self._h5py_obj

    @property
    def path(self):
        """
        The path this object points to
        """
        return self._path

    def _set_file(self, f):
        """
        set file associated with the hard link
        """
        if self._path is not None:
            self._h5py_obj = f[self._path]
        else:
            raise RuntimeError(NO_PATH)

    def __repr__(self):
        if self._path is not None:
            return "HardLink(path={path})".format(path=self.path)
        else:
            return "HardLink(h5py_obj={obj})".format(obj=self.h5py_obj)


def open(filename, registries, **kwargs):
    """
    Open a hdf5 file wrapped with h5preserve.

    Parameters
    ----------
    filename : string, or other identifier accepted by ``h5py.File``
    registries : RegistryContainer
        the collection of registries that you want to use to read from the hdf5
        file
    **kwargs
        additional keyword arguments to pass to ``h5py.File``
    """
    return H5PreserveFile(h5py.File(filename, **kwargs), registries)


def new_registry_list(*registries):
    """
    Create a new list of registries which includes builtin registries.

    Parameters
    ----------
    *registries : list of Registry
        the list of registries to be associated with this container
    """
    from .additional_registries import (
        none_python_registry, builtin_numbers_registry,
        builtin_text_registry, sequence_as_dataset_registry,
    )
    return RegistryContainer(
        none_python_registry,
        builtin_numbers_registry,
        builtin_text_registry,
        sequence_as_dataset_registry,
        *registries
    )

RECURSIVE_DUMPING_TYPES = {
    GroupContainer
}



import pytest

from h5preserve import HardLink

class TestHardLink(object):
    def test_no_path(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        link = HardLink(group)
        with pytest.raises(RuntimeError) as excinfo:
            link._set_file(h5py_file_with_group)
        assert (
            "No path defined for hard link." == str(excinfo.value)
        )

    def test_set_file(self, h5py_file_with_group):
        link = HardLink("example")
        group = h5py_file_with_group["example"]
        link._set_file(h5py_file_with_group)
        assert link.h5py_obj == group

    def test_create_with_path(self):
        link = HardLink("example")
        assert "example" == link._path
        assert link._h5py_obj is None

    def test_create_with_h5py_obj(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        link = HardLink(group)
        assert link._path is None
        assert link._h5py_obj == group

    def test_passthrough_path(self):
        path = "example"
        assert path == HardLink(path)._path

    def test_passthrough_h5py_group(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        assert HardLink(group).h5py_obj == group

    def test_passthrough_h5py_dataset(self, h5py_file_with_dataset):
        dataset = h5py_file_with_dataset["example"]
        assert HardLink(dataset).h5py_obj == dataset

    def test_passthrough_h5py_file(self, h5py_file):
        file = h5py_file
        assert HardLink(file).h5py_obj == file



import setuptools

import versioneer

DESCRIPTION_FILES = ["pypi-intro.rst"]

long_description = []
import codecs
for filename in DESCRIPTION_FILES:
    with codecs.open(filename, 'r', 'utf-8') as f:
        long_description.append(f.read())
long_description = "\n".join(long_description)


setuptools.setup(
    name = "h5preserve",
    version = versioneer.get_version(),
    packages = setuptools.find_packages(),
    install_requires = [
        "h5py",
    ],
    author = "James Tocknell",
    author_email = "aragilar@gmail.com",
    description = "Thin wrapper around h5py, inspired by camel",
    long_description = long_description,
    license = "3-clause BSD",
    keywords = "hdf5",
    url = "http://h5preserve.rtfd.org",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Topic :: Scientific/Engineering',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    cmdclass=versioneer.get_cmdclass(),
)



import pytest

from h5preserve import H5PreserveGroup, new_registry_list

class TestGroup(object):
    def test_create_group(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        h5preserve_group = H5PreserveGroup(group, new_registry_list())
        h5preserve_subgroup = h5preserve_group.create_group("subexample")
        assert h5preserve_subgroup.h5py_group == group["subexample"]

    def test_require_group_no_exist(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        h5preserve_group = H5PreserveGroup(group, new_registry_list())
        h5preserve_subgroup = h5preserve_group.require_group("subexample")
        assert h5preserve_subgroup.h5py_group == group["subexample"]

    def test_require_group_exist(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        h5preserve_group = H5PreserveGroup(
            h5py_file_with_group, new_registry_list()
        )
        h5preserve_subgroup = h5preserve_group.require_group("example")
        assert h5preserve_subgroup.h5py_group == group

    def test_roundtrip(self, h5py_file_with_group):
        group = h5py_file_with_group["example"]
        assert group == H5PreserveGroup(
            group, new_registry_list()
        ).h5py_group



import pytest

from h5preserve import RegistryContainer

class TestLoad(object):
    def test_not_loadable(self, experiment_data):
        registries = RegistryContainer()
        with pytest.raises(TypeError) as excinfo:
            registries.load(experiment_data)
        assert (
            "<class 'conftest.Experiment'> is not something that can be loaded." == str(excinfo.value)
        )


    @pytest.mark.xfail
    def test_unknown_namespace(self, tmpdir, registry_container):
        assert 0

    def test_label_not_in_namespace(
        self, h5preserve_experiment_repr, no_loader_experiment_registry
        ):
        registries = RegistryContainer(no_loader_experiment_registry)
        with pytest.raises(RuntimeError) as excinfo:
            registries.load(h5preserve_experiment_repr)
        assert (
            "Label Experiment not in namespace experiment." == str(excinfo.value)
        )

    @pytest.mark.xfail
    def test_no_suitable_loader(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_none(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_any(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_all(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_recursive(self, tmpdir, registry_container):
        assert 0

class TestFromFile(object):
    @pytest.mark.xfail
    def test_no_loadable_type(self, tmpdir, registry_container):
        assert 0
    @pytest.mark.xfail
    def test_unknown_namespace(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_recursive(self, tmpdir, registry_container):
        assert 0


class TestH5pyToH5preserve(object):
    @pytest.mark.xfail
    def test_group(self, tmpdir, registry_container):
        assert 0
    @pytest.mark.xfail
    def test_dataset(self, tmpdir, registry_container):
        assert 0

    @pytest.mark.xfail
    def test_unsupported_type(self, tmpdir, registry_container):
        assert 0
